/**
 * @file dmxcue.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Represents a DMX data cue in a queue
 * Also contains information about the scene
**/
#include "dmxcue.h"
#include <stdlib.h>




/**
 * Initiates a new list
 * @return List object, 0 on fail
**/
dmxlist_t *dmxcue_init()
{
	dmxlist_t *list = malloc(sizeof(dmxlist_t));
	if(list == 0) return 0;
	list->size  = 0;
	list->first = 0;
	/* trigger settings */
	list->trigger_type = DMXCUE_TRIGGER_DEFAULT;
	list->trigger_time = 1;
	list->trigger_fade = 0;
	return list;
}


/**
 * Clear/Reset list
 * Deletes all members
 * @param list List to clear
 * @return 1 on success
**/
int dmxcue_clear(dmxlist_t *list)
{
	if(list == 0) return 0;
	dmxpair_t *obj = list->first;
	list->first = 0;
	if(obj == 0) return 1;
	dmxpair_t *tmp;
	while(obj != 0)
	{
		tmp = obj;
		obj = obj->next;
		free(tmp);
	}
	list->size = 0;
	return 1;
}


/**
 * Cleans up dmxlist_t
 * complete destruction
 * @param list List to destroy
**/
void dmxcue_destroy(dmxlist_t *list)
{
	if(list == 0) return;
	dmxcue_clear(list);
	free(list);
}


/**
 * Adds an element to the list 
 * @param channel Channel to set
 * @param value   Value to set to
 * @param list    List object
 * @return 1 on insert, 2 on overwrite, 0 on failure
**/
int dmxcue_set(int channel, unsigned char value, dmxlist_t *list)
{
	if(list == 0) return 0;
	dmxpair_t *obj = list->first;
	dmxpair_t *prev = 0;
	/* empty list */
	if(obj == 0)
	{
		list->first = malloc(sizeof(dmxpair_t));
		list->first->channel  = channel;
		list->first->value    = value;
		list->first->next     = 0;
		/* increase list length */
		list->size++;
		return 1;
	}
	/* search if channel already exists */
	do
	{
		/* if found -> refresh */
		if(obj->channel == channel)
		{
			obj->value = value;
			return 1;
		}
		prev = obj;
		obj = obj->next;
	} while(obj != 0);
	/* if not found: append */
	prev->next = (dmxpair_t*) malloc(sizeof(dmxpair_t));
	prev->next->channel = channel;
	prev->next->value   = value;
	prev->next->next    = 0;
	list->size++;
	return 1;
}


/**
 * Sets an channel additive if existing
 * @param channel Channel to set
 * @param value   Value to ADD
 * @param list    List object
 * @return 1 on insert, 2 on overwrite, 0 on failure
**/
int dmxcue_setadd(int channel, int addvalue, dmxlist_t *list)
{
	int value = dmxcue_get_channel_value(channel, list) + addvalue;
	if(value > 255) value = 255;
	if(value < 0)   value = 0;
	return dmxcue_set(channel, (unsigned char) value, list);
}


/**
 * Unsets/Deletes a channel setting from list
 * @param channel Channel to unset
 * @param list    The list object
 * @return 1 on success
**/
int dmxcue_unset(int channel, dmxlist_t *list)
{
	if(list == 0) return 0;
	
	dmxpair_t *obj = list->first;
	dmxpair_t *pre = 0;
	if(obj == 0) return 0;
	/* search if channel already exists */
	do
	{
		/* if found -> delete */
		if(obj->channel == channel)
		{
			if(pre != 0) pre->next = obj->next;
			else list->first = obj->next;
			free(obj);
			list->size--;
			return 1;
		} 
		pre = obj;
		obj = obj->next;
	} while(obj != 0);
	return 0;
}


/**
 * Get the next element (list iteration)
 * @param this element
 * @return follower
**/
dmxpair_t *dmxcue_get_next(dmxpair_t *this)
{
	return this->next;
}


/**
 * Get a given channels object
 * @param channel Channel to search for
 * @param list    List to search in
 * @return dmxpair_t object of a channel
**/
dmxpair_t *dmxcue_get_channel_obj(int channel, dmxlist_t *list)
{
	if(list == 0) return 0;
	dmxpair_t *obj = list->first;
	if(obj == 0) return 0;
	/* search if channel already exists */
	do
	{
		/* if found -> refresh */
		if(obj->channel == channel)
		{
			return obj;
		}
		obj = obj->next;
	} while(obj != 0);
	return 0;
}


/**
 * Get a given channels object
 * @param channel Channel to search for
 * @param list    List to search in
 * @return value of the given channel
**/
unsigned char dmxcue_get_channel_value(int channel, dmxlist_t *list)
{
	dmxpair_t *p = dmxcue_get_channel_obj(channel, list);
	if(p==0) return 0;
	return p->value;
}


/**
 * Saves cue into a file
 * @param fp    File pointer to store data in
 * @param list  Data list
 * @return Number of bytes written
**/
long dmxcue_save(FILE *fp, dmxlist_t *list)
{
	long len;
	dmxpair_t end;
	dmxpair_t *p = list->first;
	/* save list */
	len = 0;
	while(p != 0)
	{
		len += fwrite(p, sizeof(dmxpair_t), 1, fp);
		p = dmxcue_get_next(p);
	}
	/* save end marker */
	end.channel = -1;
	end.value   = 0;
	len += fwrite(&end, sizeof(dmxpair_t), 1, fp);
	return len;
}


/**
 * Reads cue from a file
 * @param fp   File pointer to read data from
 * @param list  Data list
 * @return Number of bytes written
**/
long dmxcue_load(FILE *fp, dmxlist_t *list)
{
	long len, tmp;
	dmxpair_t buf;
	/* clear list to avoid appended loading */
	dmxcue_clear(list);
	/* load until end marker */
	len = 0;
	while(!feof(fp))
	{
		/* read */
		tmp = fread(&buf, sizeof(dmxpair_t), 1, fp);
		if(tmp <= 0) break;
		len += tmp;
		/* end marker reached */
		if(buf.channel == -1) break;
		/* append */
		dmxcue_set(buf.channel, buf.value, list);
	}
	return len;
}



/**
 * Checks if a cue ist empty
 * @param list    List to check
 * @return 1 if cue is empty or non-existing
**/
int dmxcue_is_empty(dmxlist_t *list)
{
	if(list == 0) return 1;
	if(list->first == 0) return 1;
	if(list->size <= 0)  return 1;
	return 0;
}
