/**
 * @file dmxgui.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Provides a ncurses gui for dmxctrl
**/
#include "dmxgui.h"
#include "gui/dmxgui_tools.h"
#include "gui/detail.h"
#include "gui/listing.h"
#include "gui/programmer.h"
#include "gui/edit.h"
#include "gui/global.h"
#include "dmxplayer.h"
#include "cmdbuf.h"



dmxgui_status_t gui;
WINDOW *wnd;
int tapstep;




/**
 * gui_show_bpminfo()
**/
void gui_show_global_info()
{
	wattron(wnd, COLOR_PAIR(5));
	/* bpm info */
	long quarter_cnt = dmxplayer_getQuarterCnt();
	char stepsign = dmxgui_get_step_sign(quarter_cnt);
	mvwprintw(wnd, 1, 2, "[%c] %3.2f bpm   ", stepsign, dmxplayer_getbpm());
	mvwprintw(wnd, 1, 20, "t=%04ld.%ld", (quarter_cnt/4), (quarter_cnt%4+1));
	/* tap info */
	if(!dmxplayer_istaptimeout()) mvwprintw(wnd, 1, 30, "Tap=%d", tapstep+1);
	else                          mvwprintw(wnd, 1, 30, "Tap=_");
	/* selected program */
	int prog = listing_getSelectedProgram();
	dmxprogram_t *p;
	p = dmxplayer_getprogram(prog);
	if(p != 0) mvwprintw(wnd, 1, 38, "A:C%02d G:%02d S:%02d   ", (prog+1), p->group, p->size);
	/* toogle color */
	wattroff(wnd, COLOR_PAIR(5));
	wattron(wnd, COLOR_PAIR(1));
	/* check if programmer is cleared */
	if( dmxplayer_is_set_prog() )
		mvwprintw(wnd, 1, 60, "[CLEAR PROG!]", (prog+1), p->group, p->size);
	else
		mvwprintw(wnd, 1, 60, "             ", (prog+1), p->group, p->size);
	wattroff(wnd, COLOR_PAIR(1));
}



/**
 * GUI MAIN RUNNING FUNCTION
 * CONTAINS MAIN LOOP
**/
void dmxgui_run(int user_channel_cnt, const char *filename)
{
	FILE *tf;
	int ret;
	/* set gui struct content */
	gui.screen_mode = DMXGUI_MODE_PROGRAMMER;
	gui.prev_key_main = 0;
	gui.is_user_input = 0;
	gui.inplen        = 0;
	gui.input[0]      = '\0';
	if(filename == 0) sprintf(gui.filename, "unnamed.set");
	tapstep = 0;
	
	/* set maximum channel */
	gui.channel_cnt = user_channel_cnt;
	if(gui.channel_cnt > 511) gui.channel_cnt = 511;

	/* init/start player */
	int dmxret = 0;
	dmxret = dmxplayer_init();
	if(dmxret != 1) {endwin(); exit(1);}

	/* load from FILE if wanted to */
	if(filename != 0)
	{
		sprintf(gui.filename, "%s", filename);
		tf = fopen(gui.filename, "rb");			
		if(tf == 0) 
		{
			fprintf(stderr, "Can not open or read file \'%s\'\n", gui.filename);
			exit(1);
		}
		dmxgui_load(tf);
		ret = dmxplayer_load(tf);
		fclose(tf);
		if(ret <= 0)
		{
			fprintf(stderr, "Error reading file \'%s\'\n", gui.filename);
			exit(1);
		}
	}

	/* init screen, disable line-buffering, enable functional keys, ... */
	dmxgui_ncurses_settings();

	/* Colors */
	dmxgui_color_init();

	/* clear own terminal's buffer */
	cmdbuf_clear();

	/* create window */
	wnd = newwin(LINES, COLS, 0, 0);
	wattrset(wnd, COLOR_PAIR(1));
	keypad(wnd, TRUE);
	/* timeout delay in ms for wgetch */
	wtimeout(wnd, 500);

	/* set selected program */
	listing_setSelectedProgram(0);
	/* init and show programmer */
	programmer_init(gui.channel_cnt);

	/* refresh window before main loop */
	while( 1 )
	{
		/* print active function */
		switch(gui.screen_mode)
		{
			case DMXGUI_MODE_GLOBAL:
				global_printer(wnd, &gui);
				break;
			case DMXGUI_MODE_EDIT:
				edit_printer(wnd, &gui);
				break;
			case DMXGUI_MODE_DETAIL:
				detail_printer(wnd, &gui);
				break;
			case DMXGUI_MODE_LISTING:
				listing_printer(wnd);
				break;
			case DMXGUI_MODE_PROGRAMMER:
			default:
				programmer_printer(wnd);
				break;
		}

		/* prints tab selection info on window title */
		dmxgui_print_tab_titles(wnd, gui.screen_mode);
		/* show bpm info */
		gui_show_global_info();

		/* refresh window */
		wmove(wnd,0,0);
		wrefresh(wnd);
	
		/* get keyevent */
		if( !dmxgui_keyevent() )
			break;
	}

	/* destroy and stop player */
	dmxplayer_destroy();

	/* clear screen before exit */	
	wclear(wnd);
	clear();
	/* exit window mode */
	endwin();
}



/**
 * dmxgui_keyevent()
 * Handles a key event
 * @return 1 on standart, 0 if program should be terminated
**/
int dmxgui_keyevent()
{
	int c;
	/* get user input */
	c = wgetch(wnd);
	if( (c == ERR) ) return 1;

	/* read a string */
	if(gui.is_user_input == 1)
	{
		/* ENTER -> Use input as value */
		if( (c==KEY_ENTER) || (c=='\n') || (c=='\r') )
		{
			c = DMXGUI_KEY_INPUT_FIN;
			gui.is_user_input = 0;
		}
		else if(c==KEY_BACKSPACE)
		{
			if(gui.inplen > 0) --gui.inplen;
			gui.input[gui.inplen] = '\0';
			return 1;
		}
		else if(gui.inplen < (DMXGUI_BUFLEN-1))
		{
			gui.input[gui.inplen]     = c;
			gui.input[++(gui.inplen)] = '\0';
			return 1;
		}
	}

	if( (c == 27) && (gui.prev_key_main == 27) ) return 0;
	/* toogle program */
	else if( (gui.screen_mode != DMXGUI_MODE_GLOBAL) && (c >= 'a') && (c <= 'z') )
	{
		dmxplayer_toogle_program(c-'a');
	}
	/* handle user input */
	else
	{
		switch(c)
		{
			/* toogle view */
			case KEY_STAB:
			case '\t':
			case KEY_F(9):
				gui.screen_mode++;
				if(gui.screen_mode > DMXGUI_MAXMODE) gui.screen_mode = 0;
				clear();
				wclear(wnd);
				break;
			case KEY_BTAB:
				gui.screen_mode--;
				if(gui.screen_mode < 0) gui.screen_mode = DMXGUI_MAXMODE;
				clear();
				wclear(wnd);
				break;
			/* time tap */
			case KEY_F(7):
			case ' ':
				tapstep = dmxplayer_timetap();
				break;
			/* time sync */
			case KEY_F(8):
				dmxplayer_timesync();
				break;

			/* global clear programmer */
			case 'C':
				dmxplayer_clear_prog();
				break;

			/* Active windows action handler */
			default:
				switch(gui.screen_mode)
				{
					case DMXGUI_MODE_GLOBAL:
						global_keyevent(c, &gui);
						break;
					case DMXGUI_MODE_EDIT:
						edit_keyevent(c, &gui);
						break;
					case DMXGUI_MODE_DETAIL:
						detail_keyevent(c, &gui);
						break;
					case DMXGUI_MODE_LISTING:
						listing_keyevent(c);
						break;
					case DMXGUI_MODE_PROGRAMMER:
						programmer_keyevent(c);
						break;
				}
		}
	}
	/* set previous key reminder */
	gui.prev_key_main = c;
	return 1;
}



/**
 * Saves dmx gui data into a file
 * @param fp    File pointer to store data in
 * @return Number of bytes written
**/
long dmxgui_save(FILE *fp)
{
	long len;
	len = 0;
	len += fwrite(&(gui.channel_cnt), sizeof(int), 1, fp);
	return len;
}



/**
 * Reads dmx gui data into a file
 * @param fp   File pointer to read data from
 * @return Number of bytes read
**/
long dmxgui_load(FILE *fp)
{
	long len;
	len = 0;
	len += fread(&(gui.channel_cnt), sizeof(int), 1, fp);
	return len;
}
