/**
 * @file cmdexec.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / 2012-2014 / All rights reserved. 
 * ---------------------------------------------------------
 * Executes an lindmxd command
**/
#include "cmdexec.h"
#include "modules/dmxmodule.h"
#include <stdio.h>
#include <syslog.h>
#include <string.h>
#include <pthread.h>

int memory[512];
int dmx_maxchannel = 0;
pthread_mutex_t mutex;


/**
 * cmdexec_init()
 * This will init the interface and also the command interpreter
 * @param maxchannel  Number of the highest DMX channel
 * @param interface   Name of the device to use
 * @param zero        if zero is set, all channels are reset on startup
 * @param ihost       The Interface's Hostname
 * @param iport       The Interface's Port
*/
int cmdexec_init(int maxchannel, const char *interface, int zero, const char *ihost, int iport)
{
	int m;
	dmx_maxchannel = maxchannel;

	/* Error while creating the interface object (e.g. no root rights, no interface found) */
	if( !dmxmodule_init(interface, dmx_maxchannel, ihost, iport) ) 
	{	
		syslog(LOG_ERR, "CAN NOT INIT DMX INTERFACE");
		return CMDEXEC_FAILURE;
	}

	/* Zero the channel memory */
	for(m=0; m<dmx_maxchannel; m++) memory[m] = 0;

	/* Set all channels to zero, if wanted */
	if(zero)
	{
		syslog(LOG_INFO, "setting all channel values to zero");
		dmxmodule_reset_channels();
	}
	syslog(LOG_INFO, "dmx interface initialisation ok");

	pthread_mutex_init(&mutex, NULL);
	return CMDEXEC_SUCCESS;
}



/**
 * cmdexec_destroy()
 * This will deinit the interface and also the command interpreter
*/
void cmdexec_destroy()
{
	dmxmodule_destroy();
	pthread_mutex_destroy(&mutex);
}



/**
 * cmdexec_do()
 * Execute a given command and give a response
 * WARNING: Use a mutex before calling!
 * cmd has to be ensured to end with \0 - use kill_lineend()
 * @param cmd        The command to execute
 * @param response   Response cmdfer (response will end on \r\n)
 * @param resp_len   Response cmdfer length
**/
int cmdexec_do(const char *cmd, char *response, int resp_len)
{
	int ret, val, chan, m;
	/* Server shut-down/stop wanted */
	if( strcasecmp("stop", cmd) == 0 ) 
	{
		/* stop the server */
		snprintf(response, resp_len, "going to stop now.\r\n");
		lindmxd_stop();
		return CMDEXEC_SUCCESS;
	}
	/* Client wants to quit */
	else if( (strcasecmp("quit", cmd) == 0) || (strcasecmp("exit", cmd) == 0) ) 
	{
		snprintf(response, resp_len, "bye\r\n");
		return CMDEXEC_CLIQUIT;
	}
	/* DMX: Full Off */
	else if(strcasecmp("fulloff", cmd) == 0) 
	{
		pthread_mutex_lock(&mutex);
			/* Set memory */
			for(m=0; m<dmx_maxchannel; m++) memory[m] = 0;
			/* Set device */
			ret = dmxmodule_reset_channels();
		pthread_mutex_unlock(&mutex);
		/* check for error */
		if(ret <= 0) 
		{
			syslog(LOG_ERR, "[ERROR] reset all channels failed!");
			snprintf(response, resp_len, "error\r\n");
			return CMDEXEC_FAILURE;
		}
		snprintf(response, resp_len, "ok\r\n");
		return CMDEXEC_SUCCESS;
	}
	/* DMX: Full On */
	else if(strcasecmp("fullon", cmd) == 0) 
	{
		pthread_mutex_lock(&mutex);
			/* Set memory */
			for(m=0; m<dmx_maxchannel; m++) memory[m] = 255;
			/* Set device */
			ret = dmxmodule_set_all(255);
		pthread_mutex_unlock(&mutex);
		/* check for error */
		if(ret <= 0) 
		{
			syslog(LOG_ERR, "[ERROR] set all channels failed!");
			snprintf(response, resp_len, "error\r\n");
			return CMDEXEC_FAILURE;
		}
		snprintf(response, resp_len, "ok\r\n");
		return CMDEXEC_SUCCESS;
	}
	/* DMX: Set every channel */
	else if ( (sscanf(cmd, "set * %d", &val) == 1) ||
	          (sscanf(cmd, "SET * %d", &val) == 1) )
	{
		if((val < 0) || (val>255))
		{
			snprintf(response, resp_len, "Invalid value - range is 0-255\r\n");
			return CMDEXEC_FAILURE;
		}
		pthread_mutex_lock(&mutex);
			/* Set memory */
			for(m=0; m<dmx_maxchannel; m++) memory[m] = val;
			/* Set device */
			ret = dmxmodule_set_all(val);
		pthread_mutex_unlock(&mutex);
		/* check for error */
		if(ret <= 0) 
		{
			syslog(LOG_ERR, "[ERROR] set all channels failed!");
			snprintf(response, resp_len, "error\r\n");
			return CMDEXEC_FAILURE;
		}
		snprintf(response, resp_len, "ok\r\n");
		return CMDEXEC_SUCCESS;
	}
	/* DMX: Set single channel value */
	else if( (sscanf(cmd, "set %d %d", &chan, &val) == 2) ||
	         (sscanf(cmd, "SET %d %d", &chan, &val) == 2) )
	{
		chan--;
		if((chan < 0) || (chan>=dmx_maxchannel))
		{
			/* invalid channel message */
			snprintf(response, resp_len, "Invalid channel - range is 1-%d\r\n", dmx_maxchannel);
			return CMDEXEC_FAILURE;
		}
		if((val < 0) || (val>255))
		{
			/* invalid value message */
			snprintf(response, resp_len, "Invalid value - range is 0-255\r\n");
			return CMDEXEC_FAILURE;
		}
		/* set channel value and give answer */
		pthread_mutex_lock(&mutex);
			memory[chan] = val;
			ret = dmxmodule_set(chan, val);
		pthread_mutex_unlock(&mutex);
		/* check for error */
		if(ret <= 0)
		{
			syslog(LOG_ERR, "[ERROR] interface error: set channel failed!");
			snprintf(response, resp_len, "error - interface unreachable\r\n");
			return CMDEXEC_FAILURE;
		}
		snprintf(response, resp_len, "ok\r\n");
		return CMDEXEC_SUCCESS;
	}
	/* DMX: get channel value */
	else if( (sscanf(cmd, "get %d", &chan) == 1) || 
	         (sscanf(cmd, "GET %d", &chan) == 1) )
	{
		chan--;
		if((chan < 0) || (chan>=dmx_maxchannel))
		{
			/* invalid channel message */
			snprintf(response, resp_len, "Invalid channel - range is 1-%d\r\n", dmx_maxchannel);
			return CMDEXEC_FAILURE;
		}
		snprintf(response, resp_len, "%d\r\n", memory[chan]);
		return CMDEXEC_SUCCESS;
	}
	/* Connection counter requested */
	else if(strcasecmp("commands", cmd) == 0) 
	{
		snprintf(response, resp_len, "commands: quit / fullon / fulloff / "
		                             "setall <value> / set <channel> <value> / "
		                             "get <channel> / stop\r\n");
		return CMDEXEC_SUCCESS;
	}
	/* No valid command provided */
	syslog(LOG_DEBUG, "received unknown command");
	strncpy(response, "command not known - type 'commands' for info\r\n", resp_len);
	return CMDEXEC_FAILURE;
}
