/**
 * @file lindmxd.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / 2012-2014 / All rights reserved. 
 * ---------------------------------------------------------
 * Here the main service is done.
**/
#include "lindmxd.h"
#include "signals.h"
#include "cmdexec.h"
#include "lindmxd_tcp.h"
#include "modules/dmxmodule.h"
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/stat.h>



/**
 * kill_lineend()
 * Kills the line end. The incoming string format does not matter: 
 * '\r', '\n', '\r\n' and '\n\r' is accepted.
 * @param str    String to modify
 * @param buflen buffer length, maximum place to go to
 * @return string length
**/
int kill_lineend(char *str, int buflen)
{
	int n = 0;
	/* look for the line/string end */
	while( n<buflen && (str[n] != '\r') && (str[n] != '\n') && (str[n] != '\0') ) ++n;
	/* place a \0 at the end */
	str[n] = '\0';
	return (n-1);
}



/**
 * lindmxd_start()
 * Start the daemon, fork processes etc.
 * @param daemon_name Name of the daemon, also of the logfile
 * @param no_fork     If nofork is set to 1, no fork is done
**/
void lindmxd_start(lindmxd_config_t *conf)
{
	int pid;

	if(conf->debug != 1)
	{
		/* fork the processes */
		pid = fork();
		/* something went wrong */
		if(pid < 0) exit(EXIT_FAILURE);
		/* this is the parent process */
		if(pid > 0) exit(EXIT_SUCCESS);

		/* set the child as session leader / returns session id */
		if(setsid() == -1)
		{
			fprintf(stderr, "%s error becoming the session leader.\n", conf->daemon_name);
			exit(0);
		}
	}

	/* set umask to get full file access */
	umask(0);

	/* Goto root directory */
	if( chdir("/") < 0 )
	{
		fprintf(stderr, "Can not toggle daemon to root directory.\n");
		exit(EXIT_FAILURE);
	}

	/* Set log message importance */
	setlogmask( LOG_UPTO(conf->log_upto) );

	/* close standart file descriptors */
	if(conf->debug != 1)
	{
		close(STDIN_FILENO);
		close(STDOUT_FILENO);
		close(STDERR_FILENO);
		/* open syslog */
		openlog(conf->daemon_name, LOG_PID, conf->log_facility);
		syslog(LOG_NOTICE, "starting lindmxd");
	}
	else
	{
		/* open syslog */
		openlog(conf->daemon_name, LOG_PID|LOG_PERROR, LOG_LOCAL1);
		syslog(LOG_NOTICE, "starting lindmxd in no_fork/debug mode");
	}

	/* Set signal destination */
	signal(SIGINT, signals_handle_sig);

	/* Init command executor and interface */
	syslog(LOG_DEBUG, "beginning interface initilization...");
	if( cmdexec_init( conf->channels, 
	                  conf->interface, 
	                  conf->zero, 
	                  conf->if_host, 
	                  conf->if_port)  ==  CMDEXEC_FAILURE )
		exit( EXIT_FAILURE );

	/* run the main servers loop */
	syslog(LOG_DEBUG, "starting network services...");
	lindmxd_run_tcp(conf->port);

	abort();
}



/**
 * lindmxd_stop()
 * This function stops the server.
*/
void lindmxd_stop()
{
	/* wait a bit */
	usleep(500*1000);
	/* clear interface */
	dmxmodule_destroy();
	usleep(500);

	/* final shutdown */
	syslog(LOG_NOTICE, "lindmxd shutting down.");

	/* Destroy components created by main */
	cmdexec_destroy();
	
	/* after running, close log */
	closelog();

	exit(EXIT_SUCCESS);
}



